import copy
import inspect

import django.db.models.options as options
import django_filters.utils
from django.utils.translation import gettext as _

from ._version import get_version
from .constants import META_REPORT_KEY, BaseExportFormat, ReportModel

options.DEFAULT_NAMES = options.DEFAULT_NAMES + (META_REPORT_KEY,)


export_format = BaseExportFormat

report_model = ReportModel


__version__ = get_version()
__all__ = [
    "ReportModel",
    "report_model",
    "BaseExportFormat",
    "export_format",
]


_verbose_field_name = copy.deepcopy(django_filters.utils.verbose_field_name)


def verbose_field_name(model, field_name):
    from .utils import get_model_property, nested_getattr

    verbose_field_name_source = (
        inspect.getsource(_verbose_field_name)
        .replace('return " ".join(names)', "return names")
        .replace("verbose_field_name", "override_verbose_field_name")
    )
    exec(verbose_field_name_source, django_filters.utils.__dict__, globals())

    if field := get_model_property(model, field_name):
        names = [
            nested_getattr(
                field, "fget.verbose_name", getattr(field, "__name__", "[invalid name]")
            )
        ]
    else:
        names = globals()["override_verbose_field_name"](model, field_name)

    return " ".join(map(_, names))


if inspect.getsource(_verbose_field_name) != inspect.getsource(verbose_field_name):
    django_filters.utils.verbose_field_name = verbose_field_name



from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

from report import report_model


class IntermediateTest(models.Model):
    name = models.CharField(max_length=200, verbose_name=_("Name"))
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name="intermediate_tests")

    @classmethod
    def report_search_fields(cls):
        return ["name"]


@report_model.register
class Test(models.Model):
    test = models.ForeignKey(IntermediateTest, on_delete=models.CASCADE, related_name="tests")
    title = models.CharField(max_length=200, verbose_name=_("Title"))

    @classmethod
    def report_search_fields(cls):
        return ["title"]



# Generated by Django 4.2.7 on 2023-12-09 09:54

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("playground", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="IntermediateTest",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=200, verbose_name="Name")),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="intermediate_tests",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="test",
            name="test",
            field=models.ForeignKey(
                default=1,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="tests",
                to="playground.intermediatetest",
            ),
            preserve_default=False,
        ),
    ]
